SystemOrganization addCategory: #'XML-Writer-Core'!

Object subclass: #XMLExternalIDWriter
	instanceVariableNames: 'publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLExternalIDWriter commentStamp: 'monty 1/19/2014 22:07' prior: 0!
This class writes public and system IDs of external entities.!

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/30/2013 01:10'!
hasExternalID
	^ self publicID notEmpty or: [self systemID notEmpty]! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'JAAyer 8/6/2010 20:16'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:53'!
publicID: aPublicId
	publicId := aPublicId! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'JAAyer 8/6/2010 20:16'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:53'!
systemID: aSystemId
	systemId := aSystemId! !

!XMLExternalIDWriter methodsFor: 'writing markup' stamp: 'monty 2/9/2014 03:38'!
writeOn: aWriter
	self publicID
		ifNotEmpty: [:id |
			aWriter
				nextPutAll: 'PUBLIC';
				space;
				writeUnescapedString: id]
		ifEmpty: [
			self systemID
				ifNotEmpty: [aWriter nextPutAll: 'SYSTEM']].

	self systemID ifNotEmpty: [:id |
		aWriter
			space;
			writeUnescapedString: id].! !

Object subclass: #XMLMarkupWriter
	instanceVariableNames: 'writer isWriting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupWriter commentStamp: 'JAAyer 7/12/2010 02:41' prior: 0!
This is a base class for markup writers. Subclasses override messages like #writePrologue and #writeEpilogue to write markup using the supplied XMLWriter object.!

XMLMarkupWriter subclass: #XMLEmbeddedMarkupWriter
	instanceVariableNames: 'hasEmbeddedMarkup hasEmbeddedMarkupWriters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEmbeddedMarkupWriter commentStamp: 'JAAyer 8/8/2010 17:29' prior: 0!
This class is a markup writer that supports embedding of child markup.!

XMLEmbeddedMarkupWriter subclass: #XMLCDataWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCDataWriter commentStamp: 'JAAyer 7/11/2010 05:02' prior: 0!
This class generates a <!![CDATA[...]]> section. If the XMLWriter object has been set to generate canonical XML, CDATA sections will not be produced and their content will instead be XML-escaped and inserted directly into the document.!

!XMLCDataWriter methodsFor: 'testing' stamp: 'monty 2/9/2014 04:19'!
escapesEmbeddedStrings
	^ self writer isCanonical! !

!XMLCDataWriter methodsFor: 'private' stamp: 'JAAyer 6/26/2010 15:08'!
writeEpilogue
	self writer isCanonical
		ifFalse: [self writer nextPutAll: ']]>']! !

!XMLCDataWriter methodsFor: 'private' stamp: 'JAAyer 6/26/2010 15:08'!
writePrologue
	self writer isCanonical
		ifFalse: [self writer nextPutAll: '<!![CDATA[']! !

XMLEmbeddedMarkupWriter subclass: #XMLCommentWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCommentWriter commentStamp: 'JAAyer 7/11/2010 04:59' prior: 0!
Generates a comment; XMLWriter>>comment: should normally suffice.!

!XMLCommentWriter methodsFor: 'private' stamp: 'JAAyer 6/22/2010 00:14'!
writeEpilogue
	self writer nextPutAll: '-->'! !

!XMLCommentWriter methodsFor: 'private' stamp: 'JAAyer 11/8/2010 18:44'!
writePrologue	
	self writer
		cr;
		nextPutAll: '<!!--'! !

XMLEmbeddedMarkupWriter subclass: #XMLDoctypeDeclarationWriter
	instanceVariableNames: 'root externalIdWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLDoctypeDeclarationWriter commentStamp: 'JAAyer 8/18/2010 21:27' prior: 0!
This class writes <!!DOCTYPE ...> declarations. You can set the root element, public ID and system ID using the messages under "accessing" before specifying any internal subset with #internalSubset: (which results in the declaration being written). If no public ID is supplied, then the declaration written will be of type "SYSTEM" rather than "PUBLIC" and will not have any Formal Public Identifier (FPI). The internal subset, if supplied, should consist of markup declarations generated with instances of XMLMarkupDeclarationWriter subclasses.!

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 1/26/2014 01:56'!
externalIDWriter
	^ externalIdWriter ifNil: [externalIdWriter := XMLExternalIDWriter new]! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID: aPublicId
	self externalIDWriter publicID: aPublicId! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root
	^ root ifNil: ['']! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root: aRootElementName
	root := aRootElementName! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID: aSystemId
	self externalIDWriter systemID: aSystemId! !

!XMLDoctypeDeclarationWriter methodsFor: 'embedding markup' stamp: 'JAAyer 10/15/2010 15:22'!
with: aStringOrCollectionOrBlock
	self writer isCanonical
		ifFalse: [^ super with: aStringOrCollectionOrBlock]! !

!XMLDoctypeDeclarationWriter methodsFor: 'embedding markup' stamp: 'JAAyer 11/8/2010 02:59'!
withInternalSubset: aStringOrCollectionOrBlock
	^ self with: aStringOrCollectionOrBlock! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:27'!
writeEmbeddedMarkupPrologue
	self writer
		nextPutAll: ' [';
		indent! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 17:56'!
writeEpilogue
	self hasEmbeddedMarkup
		ifTrue: [
			self writer
				outdent;
				cr;
				nextPut: $]].
	self writer
		nextPut: $>;
		cr.! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 1/26/2014 02:11'!
writePrologue
	self writer
		cr;
		nextPutAll: '<!!DOCTYPE';
		space;
		writeName: self root.

	self externalIDWriter hasExternalID
		ifTrue: [
			self writer space.
			self externalIDWriter writeOn: self writer].! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/14/2014 01:34'!
embedBlock: aBlock
	aBlock cull: self writer! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 12:34'!
embedCollection: aCollection
	aCollection do: [:each | self embedValue: each]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:30'!
embedString: aString
	self prepareForEmbeddedMarkup.

	self escapesEmbeddedStrings
		ifTrue: [self writer writeEscapedContent: aString]
		ifFalse: [self writer nextPutAll: aString]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:52'!
embedValue: aValue
	aValue isString
		ifTrue: [^ self embedString: aValue].
	aValue isCollection
		ifTrue: [^ self embedCollection: aValue].
	aValue isBlock
		ifTrue: [^ self embedBlock: aValue].

	self error: 'Value object must be a string, collection or block'.! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 2/9/2014 04:19'!
escapesEmbeddedStrings
	^ false! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'JAAyer 7/26/2010 14:55'!
hasEmbeddedMarkup
	^ hasEmbeddedMarkup ifNil: [hasEmbeddedMarkup := false]! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'JAAyer 7/26/2010 16:49'!
hasEmbeddedMarkupWriters
	^ hasEmbeddedMarkupWriters ifNil: [hasEmbeddedMarkupWriters := false]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/16/2014 23:00'!
newMarkupBeingWritten
	self isWriting
		ifTrue: [self prepareForEmbeddedMarkup]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/16/2014 23:00'!
newMarkupWriterBeingPushed
	self isWriting
		ifTrue: [
			self prepareForEmbeddedMarkup.
			hasEmbeddedMarkupWriters := true]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/14/2014 01:23'!
prepareForEmbeddedMarkup
	self hasEmbeddedMarkup
		ifFalse: [
			hasEmbeddedMarkup := true.
			self writeEmbeddedMarkupPrologue]! !

!XMLEmbeddedMarkupWriter methodsFor: 'embedding markup' stamp: 'monty 10/5/2014 20:22'!
with: aStringOrCollectionOrBlock
	self
		startWriting;
		writeBody: aStringOrCollectionOrBlock;
		finishWriting! !

!XMLEmbeddedMarkupWriter methodsFor: 'writing markup' stamp: 'monty 1/14/2014 00:22'!
write
	self isWriting
		ifFalse: [self with: nil]! !

!XMLEmbeddedMarkupWriter methodsFor: 'embedding markup' stamp: 'monty 10/5/2014 15:46'!
writeBody: aStringOrCollectionOrBlock
	aStringOrCollectionOrBlock
		ifNotNil: [self embedValue: aStringOrCollectionOrBlock]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:27'!
writeEmbeddedMarkupPrologue! !

XMLEmbeddedMarkupWriter subclass: #XMLTagWriter
	instanceVariableNames: 'name attributes xmlnsDeclarations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLTagWriter commentStamp: 'monty 1/13/2014 23:32' prior: 0!
This class generates XML tags for elements. You can specifiy the attributes of the start tag using the #attribute* messages, and you can add xmlns: namespace declarations using the #xmlns* messages. If you do not send #with:, or if you do but the argument is nil or a block that generates no markup, then only a single, empty tag in the form of "<empty />" will be generated, unless the writer object that created the tag writer object is configured to generate canonical XML, in which case an empty start/end tag pair like "<empty></empty>" will be generated.

You can supply child markup content using #with:. Namespace declarations will be ignored if a parent element has equivalent declarations (and they were thus already generated).!

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:54'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsent: aBlock
	^ self attributes at: aName ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributes at: aName ifAbsentPut: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName put: aValue
	^ self attributes at: aName put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 12/6/2010 22:55'!
attributes
	^ attributes ifNil: [attributes := OrderPreservingDictionary defaultValue: '']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 23:22'!
attributes: aDictionaryOrArray
	(attributes := OrderPreservingDictionary newFrom: aDictionaryOrArray) defaultValue: ''! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 2/9/2014 04:19'!
escapesEmbeddedStrings
	^ true! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/6/2014 23:30'!
hasAttributes
	^ self attributes notEmpty! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/6/2014 23:30'!
includesAttribute: anAttribute
	^ self attributes includesKey: anAttribute! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:09'!
name
	^ name ifNil: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 16:43'!
name: aName
	name := aName! !

!XMLTagWriter methodsFor: 'initialization' stamp: 'monty 10/6/2014 20:10'!
setName: aName attributes: anAttributeDictionary xmlnsDeclarations: anXmlnsDictionary
	name := aName.
	attributes := anAttributeDictionary.
	xmlnsDeclarations := anXmlnsDictionary.! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 2/20/2014 03:24'!
writeAttributes
	attributes ifNotNil: [
		self
			writeKeysAndValuesFrom: attributes
			with: [:attributeName :attributeValue |
				self writer
					writeName: attributeName
					escapedContentValue: attributeValue]]! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:27'!
writeEmbeddedMarkupPrologue
	self writer
		nextPut: $>;
		indent! !

!XMLTagWriter methodsFor: 'private' stamp: 'JAAyer 10/20/2010 19:42'!
writeEndTag
	self writer
		nextPutAll: '</';
		writeName: self name;
		nextPut: $>! !

!XMLTagWriter methodsFor: 'private' stamp: 'JAAyer 10/20/2010 19:42'!
writeEpilogue
	self hasEmbeddedMarkup
		ifTrue: [
			self writer outdent.
			self hasEmbeddedMarkupWriters
				ifTrue: [self writer cr].
			self writeEndTag]
		ifFalse: [
			self writer isCanonical
				ifTrue: [
					self writer nextPut: $>.
					self writeEndTag]
				ifFalse: [
					self writer
						space;
						nextPutAll: '/>']].! !

!XMLTagWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 12:38'!
writeKeysAndValuesFrom: aDictionary with: aTwoArgumentBlock
	self writer isCanonical
		ifTrue: [
			aDictionary keys asSortedCollection do: [:each |
				aTwoArgumentBlock value: each value: (aDictionary at: each)]]
		ifFalse: [aDictionary keysAndValuesDo: aTwoArgumentBlock]! !

!XMLTagWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 16:33'!
writePrologue
	self writer
		cr;
		nextPut: $<;
		writeName: (self name ifNil: ['']).
	self
		writeXmlnsDeclarations;
		writeAttributes.! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 10/6/2014 23:59'!
writeXmlnsDeclarations
	xmlnsDeclarations ifNil: [^ self].

	self
		writeKeysAndValuesFrom: xmlnsDeclarations
		with: [:prefix :uri |
			self writer
				writeName:
					(prefix
						ifEmpty: ['xmlns']
						ifNotEmpty: ['xmlns:', prefix])
				escapedContentValue: uri].! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 20:59'!
xmlns
	"default namespace"
	^ self xmlnsDeclarations at: ''! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlns: aUri
	"set default namespace"
	self xmlnsDeclarations at: '' put: aUri! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix
	^ self xmlnsDeclarations at: aPrefix ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix ifAbsent: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 22:29'!
xmlnsAt: aPrefix ifAbsentPut: aUriBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsentPut: aUriBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix put: aValue
	^ self xmlnsDeclarations at: aPrefix put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 23:27'!
xmlnsDeclarations
	^ xmlnsDeclarations ifNil: [xmlnsDeclarations := OrderPreservingDictionary new: 5 withDefaultValue: '']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 23:22'!
xmlnsDeclarations: aDictionaryOrArray
	"set xmlns decls using a dictionary or array of prefix/uri associations.
	empty prefix is default xmlns."
	(xmlnsDeclarations := OrderPreservingDictionary newFrom: aDictionaryOrArray)  defaultValue: ''! !

XMLMarkupWriter subclass: #XMLMarkupDeclarationWriter
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupDeclarationWriter commentStamp: 'JAAyer 8/18/2010 21:29' prior: 0!
This is a generic superclass for classes that generate DTD markup declarations--ELEMENT, ATTLIST, ENTITY and so on.!

XMLMarkupDeclarationWriter subclass: #XMLAttributeDeclarationWriter
	instanceVariableNames: 'attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLAttributeDeclarationWriter commentStamp: 'monty 10/5/2014 18:03' prior: 0!
Writes ATTLIST declarations for attributes.!

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute: aName
	attribute := aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ATTLIST'! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:17'!
defaultPragma: aDefaultPragma
	defaultPragma := aDefaultPragma! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:17'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element
	^ self name! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element: aName
	self name: aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:51'!
type
	^ type ifNil: [type := 'CDATA']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:30'!
type: aType
	type := aType! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:30'!
writeBody
	self writer
		writeName: self attribute;
		space;
		writeName: self type.

	self defaultPragma
		ifNotEmpty: [:pragma |
			self writer space.
			(pragma first == $#)
				ifFalse: [self writer nextPut: $#].
			self writer nextPutAll: pragma].

	self defaultValue
		ifNotEmpty: [:value |
			self writer
				space;
				writeEscapedContentString: value].! !

XMLMarkupDeclarationWriter subclass: #XMLElementDeclarationWriter
	instanceVariableNames: 'contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLElementDeclarationWriter commentStamp: 'monty 10/5/2014 18:04' prior: 0!
Writes ELEMENT declarations. 
!

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:52'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 15:11'!
contentModel: aContentModelString
	contentModel := aContentModelString! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ELEMENT'! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element
	^ self name! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element: aName
	self name: aName! !

!XMLElementDeclarationWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 18:06'!
writeBody
	self writer nextPutAll: self contentModel! !

XMLMarkupDeclarationWriter subclass: #XMLEntityDeclarationWriter
	instanceVariableNames: 'replacement ndata externalIdWriter isParameter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEntityDeclarationWriter commentStamp: 'monty 10/5/2014 18:04' prior: 0!
Writes general and parameter ENTITY declarations.!

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/20/2013 23:41'!
declarationType
	^ self isParameter
		ifTrue: ['ENTITY %']
		ifFalse: ['ENTITY']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 1/26/2014 01:56'!
externalIDWriter
	^ externalIdWriter ifNil: [externalIdWriter := XMLExternalIDWriter new]! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/13/2013 19:36'!
isParameter
	^ isParameter ifNil: [isParameter := false]! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/13/2013 19:36'!
isParameter: aBoolean
	isParameter := aBoolean! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:45'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:37'!
ndata: aNotationName
	ndata := aNotationName! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID: aPublicId
	self externalIDWriter publicID: aPublicId! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:44'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:35'!
replacement: aReplacement
	replacement := aReplacement! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID: aSystemId
	self externalIDWriter systemID: aSystemId! !

!XMLEntityDeclarationWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:30'!
writeBody
	self replacement
		ifNotEmpty: [self writer writeEscapedEntityValueString: self replacement]
		ifEmpty: [
			self externalIDWriter writeOn: self writer.
			(self isParameter not and: [self ndata notEmpty])
				ifTrue: [
					self writer
						space;
						nextPutAll: 'NDATA';
						space;
						writeName: self ndata]]! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:26'!
declarationType
	self subclassResponsibility! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:43'!
name
	^ name ifNil: ['']! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 17:02'!
name: aName
	name := aName! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'JAAyer 6/24/2010 15:26'!
writeEpilogue
	self writer nextPut: $>! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'JAAyer 7/27/2010 08:26'!
writePrologue
	self writer
		cr;
		nextPutAll: '<!!';
		writeName: self declarationType;
		space;
		writeName: self name;
		space! !

XMLMarkupDeclarationWriter subclass: #XMLNotationDeclarationWriter
	instanceVariableNames: 'externalIdWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLNotationDeclarationWriter commentStamp: 'monty 10/5/2014 18:04' prior: 0!
Writes NOTATION declarations.!

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'NOTATION'! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 1/26/2014 01:56'!
externalIDWriter
	^ externalIdWriter ifNil: [externalIdWriter := XMLExternalIDWriter new]! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID: aPublic
	self externalIDWriter publicID: aPublic! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID: aSystemId
	self externalIDWriter systemID: aSystemId! !

!XMLNotationDeclarationWriter methodsFor: 'private' stamp: 'monty 1/26/2014 02:11'!
writeBody
	self externalIDWriter writeOn: self writer! !

!XMLMarkupWriter class methodsFor: 'instance creation' stamp: 'JAAyer 10/15/2010 15:19'!
writer: aWriter
	^ self new writer: aWriter! !

!XMLMarkupWriter methodsFor: 'writing markup' stamp: 'monty 10/5/2014 20:22'!
finishWriting
	self hasEmbeddedMarkupWriters
		ifTrue: [self writer writeUnwrittenMarkupWriterEmbeddedIn: self].
	self writeEpilogue.
	isWriting := false.

	self writer popActiveMarkupWriter.! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 1/14/2014 00:18'!
hasEmbeddedMarkup
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/4/2014 04:21'!
hasEmbeddedMarkupWriters
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 1/13/2014 23:55'!
isEmbedded
	^ self writer hasEmbeddedMarkupWriter: self! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 1/13/2014 23:42'!
isWriting
	^ isWriting ifNil: [isWriting := false]! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 1/16/2014 23:00'!
newMarkupBeingWritten
	self write! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 1/16/2014 23:00'!
newMarkupWriterBeingPushed
	self write! !

!XMLMarkupWriter methodsFor: 'printing' stamp: 'JAAyer 7/21/2010 09:26'!
printOn: aStream
	self writer printOn: aStream! !

!XMLMarkupWriter methodsFor: 'writing markup' stamp: 'monty 10/5/2014 15:43'!
startWriting
	isWriting := true.
	self writePrologue.! !

!XMLMarkupWriter methodsFor: 'writing markup' stamp: 'monty 10/5/2014 20:22'!
write
	self
		startWriting;
		writeBody;
		finishWriting! !

!XMLMarkupWriter methodsFor: 'writing markup' stamp: 'JAAyer 7/26/2010 12:12'!
writeBody! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writeEpilogue! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writePrologue! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'JAAyer 6/18/2010 22:05'!
writer
	^ writer! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 15:19'!
writer: aWriter
	writer := aWriter! !

XMLMarkupWriter subclass: #XMLPIWriter
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPIWriter commentStamp: 'JAAyer 7/11/2010 05:11' prior: 0!
This class generates processing instructions.!

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
data
	^ data ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:39'!
data: aString
	data := aString! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
target
	^ target ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 6/18/2010 22:44'!
target: aTarget
	target := aTarget! !

!XMLPIWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 18:19'!
writeBody
	self writer
		space;
		nextPutAll: self data.! !

!XMLPIWriter methodsFor: 'private' stamp: 'JAAyer 6/24/2010 15:26'!
writeEpilogue
	self writer nextPutAll: '?>'! !

!XMLPIWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 14:13'!
writePrologue
	self writer
		cr;
		nextPutAll: '<?';
		writeName: self target! !

XMLPIWriter subclass: #XMLXmlDeclarationWriter
	instanceVariableNames: 'encoding version standalone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLXmlDeclarationWriter commentStamp: '<historical>' prior: 0!
This class generates <?xml ...?> declarations.!

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:11'!
encoding
	^ encoding ifNil: [encoding := 'UTF-8']! !

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:13'!
encoding: anEncoding
	encoding := anEncoding! !

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:43'!
standalone
	^ standalone ifNil: [standalone := false]! !

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:12'!
standalone: aBoolean
	standalone := aBoolean! !

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:14'!
target
	^ 'xml'! !

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:11'!
version
	^ version ifNil: [version := '1.0']! !

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:12'!
version: aVersion
	version := aVersion! !

!XMLXmlDeclarationWriter methodsFor: 'accessing' stamp: 'monty 2/9/2014 03:41'!
writeBody
	self writer
		writeName: 'version' unescapedValue: self version asString;
		space;
		writeName: 'encoding' unescapedValue: self encoding.

	self standalone
		ifTrue: [
			self writer
				space;
				writeName: 'standalone' unescapedValue: 'yes']! !

Object subclass: #XMLWriter
	instanceVariableNames: 'stream markupWriterStack currentIndentLevel indentString lineBreak isCanonical omitsLineBreaks isEscaping'
	classVariableNames: 'ContentEscapes EntityValueEscapes'
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLWriter commentStamp: '<historical>' prior: 0!
This class allows you to generate well-formed XML documents using an API similar to Seaside's canvas and tag brush API.

Markup can be created by sending an instance any of the messages under "writing markup." The messages that take no arguments return an instance of an XMLMarkupWriter subclass. These objects can be configured with messages like #name: and #attributeAt:put:. Sending them #write or a specific writing message that accepts markup to embed within (the generic #with: message) causes the markup writer's configuration to be committed and its markup to be written to the output stream. For markup writers that can have embedded markup, the argument to the writing message can be a string, collection (of strings and blocks), a block, or nil. The block passed in can optionally take an argument, which will be the XMLWriter object that created the markup writer object. Within the block you can generate additional, child markup that will be contained by the parent. Creating additional non-embedded markup will cause any unwritten markup from a previous markup writer to be written, as will sending an XMLWriter or XMLMarkupWriter #contents, #printOn:, or the aforementioned #write.

Here is an example that you can highlight and evaluate with cmd-p to see the result:

	| writer |
	writer := XMLWriter new.
	writer
		enablePrettyPrinting;
		xml;
		comment: 'an example'.
	writer tag
		name: 'example:user';
		xmlnsAt: 'example' put: 'urn://example';
		with: [
			writer
				tag: 'username' with: 'jsmith';
				tag: 'real-name' with: 'John Smith';
				tag: 'age' with: '25';
				tag: 'id' with: '101']

Since the last message sent (#tag:with:) is an explicit serialization message, #write does not need to be sent. However, if you don't end with a serialization message, you must end with #write:

	| writer |
	writer := XMLWriter new.
	writer tag
		name: 'emptyTag';
		attributeAt: 'a' put: 'one';
		write.

Sending #write to the XMLWriter or markup writer object has the same effect. A simple way to avoid having to remember to send #write is to generate all markup within a top-level #writeWith: block, which will do it for you automatically:

	XMLWriter new
		writeWith: [:writer |
			writer tag
				name: 'emptyTag';
				attributeAt: 'a' put: 'value'].!

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 2/9/2014 03:28'!
initialize
	"self initialize"

	super initialize.
	self
		initializeContentEscapes;
		initializeEntityValueEscapes! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 2/9/2014 03:29'!
initializeContentEscapes
	(ContentEscapes := Array new: 128)
		at: $& asciiValue put: '&amp;';
		at: $" asciiValue put: '&quot;';
		at: $< asciiValue put: '&lt;';
		at: $> asciiValue put: '&gt;';
		at: Character cr asciiValue put: #LineBreak;
		at: Character lf asciiValue put: #LineBreak.! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 2/9/2014 03:35'!
initializeEntityValueEscapes
	(EntityValueEscapes := Array new: 128)
		at: $" asciiValue put: '&#34;';
		at: $% asciiValue put: '&#37;';
		at: Character cr asciiValue put: #LineBreak;
		at: Character lf asciiValue put: #LineBreak.! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'JAAyer 7/27/2010 13:01'!
new
	^ self on: (String new: 512) writeStream! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'JAAyer 7/27/2010 13:01'!
on: aStream
	^ self basicNew initialize
		stream: aStream! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 7/21/2010 01:09'!
% aParameterEntityReference
	^ self paramEntityRef: aParameterEntityReference! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 7/21/2010 01:08'!
& anEntityReference
	^ self entityRef: anEntityReference! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 2/9/2014 03:57'!
activeMarkupWriter
	"for speed"
	^ self markupWriterStack size > 0
		ifTrue: [markupWriterStack at: markupWriterStack size]
		ifFalse: [nil]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 10/5/2014 20:48'!
activeTagWriter
	self markupWriterStack reverseDo: [:each |
		(each class == XMLTagWriter)
			ifTrue: [^ each]].
	^ nil.
	! !

!XMLWriter methodsFor: 'printing' stamp: 'monty 1/14/2014 02:25'!
asString
	"for compatibility"
	^ self printString! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 1/13/2014 23:10'!
atStart
	^ stream position isZero! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
attributeDeclaration
	^ self pushNewMarkupWriter: XMLAttributeDeclarationWriter! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
cdata
	^ self pushNewMarkupWriter: XMLCDataWriter! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 10/15/2010 15:21'!
cdata: aString
	self cdata with: aString! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:10'!
close
	self write.
	stream close.! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
comment
	^ self pushNewMarkupWriter: XMLCommentWriter! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 10/15/2010 15:21'!
comment: aString
	self comment with: aString! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 1/13/2014 23:10'!
contents
	self write.
	^ stream contents.! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 8/6/2010 23:55'!
cr
	(self omitsLineBreaks or: [self atStart])
		ifTrue: [^ self].

	self nextPutAll: self lineBreak.
	self isCanonical
		ifFalse: [self writeIndent].! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 7/22/2010 10:21'!
currentIndentLevel
	^ currentIndentLevel ifNil: [currentIndentLevel := 0].! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
doctype
	^ self pushNewMarkupWriter: XMLDoctypeDeclarationWriter! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
elementDeclaration
	^ self pushNewMarkupWriter: XMLElementDeclarationWriter! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 7/22/2010 12:55'!
enablePrettyPrinting
	self indentString: '    '.
	self omitsLineBreaks: false.! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
entityDeclaration
	^ self pushNewMarkupWriter: XMLEntityDeclarationWriter! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/16/2014 23:03'!
entityRef: aName
	self
		newMarkup;
		nextPut: $&;
		writeName: aName;
		nextPut: $;! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/14/2014 01:07'!
hasEmbeddedMarkupWriter: aMarkupWriter
	^ self activeMarkupWriter == aMarkupWriter
		and: [self markupWriterStack size > 1]! !

!XMLWriter methodsFor: 'testing' stamp: 'JAAyer 6/24/2010 15:34'!
hasTrailingWhitespace
	| lastChar |

	^ (lastChar := self lastChar) notNil
		and: [lastChar isSeparator]! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 7/22/2010 10:21'!
indent
	currentIndentLevel := self currentIndentLevel + 1! !

!XMLWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 20:48'!
indentString
	^ indentString ifNil: [indentString := '']! !

!XMLWriter methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 10:21'!
indentString: aString
	currentIndentLevel := 0.
	indentString := aString.! !

!XMLWriter methodsFor: 'testing' stamp: 'JAAyer 6/20/2010 20:43'!
isCanonical
	^ isCanonical ifNil: [isCanonical := false]! !

!XMLWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 20:43'!
isCanonical: aBoolean
	isCanonical := aBoolean! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 2/9/2014 03:47'!
isEscaping
	^ isEscaping ifNil: [isEscaping := true]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 2/9/2014 03:47'!
isEscaping: aBoolean
	isEscaping := aBoolean! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:10'!
lastChar
	^ self atStart
		ifTrue: [nil]
		ifFalse: [stream peekLast asCharacter]! !

!XMLWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 16:15'!
lineBreak
	^ lineBreak ifNil: [lineBreak := String cr]! !

!XMLWriter methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:01'!
lineBreak: aLineBreakCharOrString
	lineBreak := aLineBreakCharOrString asString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:45'!
markupWriterStack
	^ markupWriterStack ifNil: [markupWriterStack := OrderedCollection new]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/17/2014 00:23'!
newMarkup
	self activeMarkupWriter
		ifNotNil: [:activeMarkupWriter | activeMarkupWriter newMarkupBeingWritten]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:11'!
nextPut: aCharacter
	stream nextPut: aCharacter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:11'!
nextPutAll: aString
	stream nextPutAll: aString! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
notationDeclaration
	^ self pushNewMarkupWriter: XMLNotationDeclarationWriter! !

!XMLWriter methodsFor: 'testing' stamp: 'JAAyer 7/22/2010 12:55'!
omitsLineBreaks
	^ omitsLineBreaks ifNil: [omitsLineBreaks := true]! !

!XMLWriter methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 12:55'!
omitsLineBreaks: aBoolean
	omitsLineBreaks := aBoolean! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 7/22/2010 10:21'!
outdent
	currentIndentLevel := self currentIndentLevel - 1! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/16/2014 23:03'!
paramEntityRef: aName
	self
		newMarkup;
		nextPut: $%;
		writeName: aName;
		nextPut: $;! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 7/21/2010 01:11'!
pcdata: aString
	self string: aString! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
pi
	^ self pushNewMarkupWriter: XMLPIWriter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/17/2014 02:44'!
popActiveMarkupWriter
	self markupWriterStack ifNotEmpty: [:stack | stack removeLast]! !

!XMLWriter methodsFor: 'printing' stamp: 'JAAyer 10/24/2010 15:12'!
printOn: aStream
	aStream nextPutAll: self contents.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/19/2014 21:58'!
pushNewMarkupWriter: aMarkupWriterClass
	| newMarkupWriter |

	newMarkupWriter := aMarkupWriterClass writer: self.
	self activeMarkupWriter
		ifNotNil: [:activeMarkupWriter |
			activeMarkupWriter newMarkupWriterBeingPushed].

	^ self markupWriterStack addLast: newMarkupWriter.! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/16/2014 23:04'!
raw: aString
	self
		newMarkup;
		nextPutAll: aString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:11'!
space
	(self isCanonical and: [self hasTrailingWhitespace])
		ifFalse: [stream space]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:14'!
stream
	^ stream! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 6/18/2010 22:27'!
stream: aStream
	stream := aStream! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 2/9/2014 04:30'!
string: aString
	self
		newMarkup;
		writeEscapedContent: aString.! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
tag
	^ self pushNewMarkupWriter: XMLTagWriter! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 6/26/2010 21:47'!
tag: aName
	^ self tag name: aName! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 10/6/2014 23:36'!
tag: aName attributes: aDictionaryOrArray
	^ (self tag: aName)
		attributes: aDictionaryOrArray! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 11/8/2010 02:53'!
tag: aName attributes: aDictionaryOrArray with: aStringOrCollectionOrBlock
	^ (self tag: aName)
		attributes: aDictionaryOrArray;
		with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing markup' stamp: 'JAAyer 11/8/2010 02:54'!
tag: aName with: aStringOrCollectionOrBlock
	^ (self tag: aName) with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 10/5/2014 20:25'!
write
	self activeMarkupWriter
		ifNotNil: [:activeMarkupWriter |
			activeMarkupWriter write.
			(activeMarkupWriter isWriting)
				ifTrue: [activeMarkupWriter finishWriting]]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 03:48'!
write: aString escapedWith: anEscapeTable
	| asciiValue replacement |

	self isEscaping
		ifFalse: [^ self nextPutAll: aString].

	aString do: [:each |
		((asciiValue := each asciiValue) > 0
			and: [asciiValue <= anEscapeTable size
				and: [(replacement := anEscapeTable at: asciiValue) notNil]])
			ifTrue: [
				stream nextPutAll:
					(replacement == #LineBreak
						ifTrue: [self lineBreak]
						ifFalse: [replacement])]
			ifFalse: [stream nextPut: each]]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:38'!
writeEscapedContent: aString
	self write: aString escapedWith: ContentEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:36'!
writeEscapedContentString: aString
	self
		nextPut: $";
		write: aString escapedWith: ContentEscapes;
		nextPut: $"! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:30'!
writeEscapedEntityValueString: aString
	self
		nextPut: $";
		write: aString escapedWith: EntityValueEscapes;
		nextPut: $"! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 7/22/2010 10:20'!
writeIndent
	self currentIndentLevel timesRepeat: [self nextPutAll: self indentString]! !

!XMLWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 22:43'!
writeName: aString
	self nextPutAll: aString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:31'!
writeName: aName escapedContentValue: aValue
	self
		writeName: aName
		value: aValue
		isEscapedContent: true! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:31'!
writeName: aName unescapedValue: aValue
	self
		writeName: aName
		value: aValue
		isEscapedContent: false! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:31'!
writeName: aName value: aValue isEscapedContent: aBoolean
	(self atStart or: [self hasTrailingWhitespace])
		ifFalse: [stream space].

	self
		writeName: aName;
		nextPut: $=.
	aBoolean
		ifTrue: [self writeEscapedContentString: aValue]
		ifFalse: [self writeUnescapedString: aValue].! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/9/2014 04:34'!
writeUnescapedString: aString
	| quote |

	quote := (aString includes: $") ifTrue: [$'] ifFalse: [$"].

	self nextPut: quote.
	aString do: [:each |
		each == quote
			ifFalse: [self nextPut: each]].
	self nextPut: quote.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/26/2014 01:30'!
writeUnwrittenMarkupWriterEmbeddedIn: aMarkupWriter
	self activeMarkupWriter
		ifNotNil: [:activeMarkupWriter |
			activeMarkupWriter == aMarkupWriter
				ifFalse: [activeMarkupWriter write]]! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/14/2014 01:34'!
writeWith: aBlock
	aBlock cull: self.
	self write.! !

!XMLWriter methodsFor: 'writing markup' stamp: 'monty 1/13/2014 23:52'!
xml
	^ self pushNewMarkupWriter: XMLXmlDeclarationWriter! !
XMLWriter initialize!
